package CVSMonitor::Plugin;

# This module provides the beginnings of plug-in support for CVSMonitor,
# which will mainly be used to support various file type viewers.

use strict;
use UNIVERSAL 'isa';
use File::Find::Rule;

# Are we loaded?
use vars qw{$loaded};
BEGIN {
	$loaded = {
		general => 0,
		viewer => 0,
		};
}

# Initialises the Plug-in subsystem.
# We are passed a copy of the config.
sub load {
	my $class = shift;
	my $Config = isa( $_[0], 'CVSMonitor::Config' )
		? shift : return undef;
	my $type = shift || 'all';
	return 1 if $loaded->{$type};

	# Get the plug-ins
	my $plugins = $Config->{plugins} || {};

	# Load each of the plugins
	foreach my $module ( keys %$plugins ) {
		# Skip if the type is loaded, or not wanted
		next unless ( $type eq $plugins->{$module} or $type eq 'all' );
		next if $loaded->{$plugins->{$module}};

		# Load the class
		unless ( Class::Autouse->load( $module ) ) {
			return $class->_error( "Error loading plug-in module '$module'" );
		}

		# Call the register method for the plug-in to register itself.
		unless ( $module->register() ) {
			return $class->_error( "Plug-in module '$module' returned an error while registering" );
		}
	}

	# Mark the type as loaded
	if ( $type eq 'all' ) {
		$loaded->{general} = 1;
		$loaded->{viewer} = 1;
	} else {
		$loaded->{$type} = 1;
	}
}

1;

__END__

# Copyright (C) 2002-2004 Adam Kennedy
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Should you wish to utilise this software under a different licence, 
# please contact the author.
