package CVSMonitor::Statistics;

# This package helps move some large, complex statistics related
# code out into a seperate module, so we don't have to load it all 
# the time.

use strict;
use UNIVERSAL 'isa';

# Generate 'blame' data for a File.
sub fileBlame {
	my $class = shift;
	my $File = shift;
	return undef unless isa( $File, 'CVSMonitor::MetaData::File' );

	# Get the most recent HEAD version
	my $Version = $File->getRecentHeadVersion();
	return undef unless $Version;

	$class->versionBlame( $Version );
}

# Generate blame data for a Version.
# Blame data consists of a list of the Authors that have contributed to a version,
# and how many lines they are responsible for.
# The data is returned as a Hash-of-Hashes, suitable for further processing.
sub versionBlame {
	my $class = shift;
	my $Version = shift;
	return undef unless isa( $Version, 'CVSMonitor::MetaData::Version' );

	# Get the Annotation object
	my $Annotation = $Version->Annotation or return undef;

	# Turn it into our prefered format
	my $stats = $Annotation->stats or return undef;
	my $authors = $stats->{authors};
	my %blame = ();
	foreach ( keys %$authors ) {
		$blame{$_}->{lines} = $authors->{$_};
	}

	\%blame;
}

1;

__END__

# Copyright (C) 2002-2004 Adam Kennedy
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Should you wish to utilise this software under a different licence, 
# please contact the author.
