package CVSMonitor::VersionNumber;

# This package implements some math for deep version numbers,
# as they exist in CVS.

use strict;
use UNIVERSAL 'isa';




#####################################################################
# Constructor and Friends

sub new {
	my $class = shift;
	my $revision = shift;

	# Check the version
	return undef unless length $revision;
	return undef unless $revision =~ /^[\d.]+$/;

	# Get the revision parts
	my $parts = [ split /\./, $revision ];

	# Create and retunr the object
	bless $parts, $class;
}

# Create a copy of ourself
sub copy {
	my $self = shift;
	return undef unless ref $self;
	my $class = ref $self;

	my $copy = [ $self->parts ];
	bless $copy, $class;
}





#####################################################################
# Access Methods

sub revision { join '.', @{$_[0]} } 
sub parts { @{$_[0]} }
sub elements { scalar @{$_[0]} }





#####################################################################
# Main Methods

# Find the next revision
sub next {
	# Get the object and return type
	my $either = shift;
	my $return_object = 0;
	my $self = undef;
	if ( isa( $self, 'CVSMonitor::VersionNumber' ) ) {
		$return_object = 1;
		$self = $either->copy; # Always work on a copy
	} else {
		$self = $either->new( shift ) or return undef;
	}

	# Add 1 to the last part of the copy
	$self->[ $#$self ]++;

	# Return the object or string version as needed
	$return_object ? $self : $self->revision;
}

# Get the previous revision
sub previous {
	# Get the object and return type
	my $either = shift;
	my $return_object = 0;
	my $self = undef;
	if ( isa( $self, 'CVSMonitor::VersionNumber' ) ) {
		$return_object = 1;
		$self = $either->copy; # Always work on a copy
	} else {
		$self = $either->new( shift ) or return undef;
	}

	# Calculate the previous version
	if ( $self->[-1] == 1 ) {
		# If we are the first possible version, return 0
		if ( scalar @$self == 2 ) {
			return 0;
		}

		# Remove the last two version increments
		pop @$self; pop @$self;
	} else {
		$self->[ $#$self ]--;
	}

	# Return the object or string version as needed
	$return_object ? $self : $self->revision;
}

1;

__END__

# Copyright (C) 2002-2004 Adam Kennedy
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Should you wish to utilise this software under a different licence, 
# please contact the author.
